import os
from pathlib import Path
import shutil
import subprocess
import tempfile
from typing import List

from celery.utils.log import get_task_logger

from auto_transcoder.model import MediaDTO

logger = get_task_logger(__name__)

async def transcode_media(media: MediaDTO, recycle_bin: Path | None = None) -> List[MediaDTO] | None:
    file_path = media.paths[0]

    temp_path = Path(tempfile.mkstemp()[1])
    
    full_command = ['ffmpeg', '-i', "-",
                    '-vf', 'scale=\'min(1920,iw)\':-1:flags=lanczos',
                    '-c:v', 'libsvtav1', '-crf', '30', '-preset', '6', '-g', '240', '-pix_fmt', 'yuv420p10le',
                    '-c:a', 'libopus', '-b:a', '128k', '-ac', '2',
                    '-c:s', 'webvtt',
                    '-map_chapters', '-1', '-map_metadata', '-1',
                    '-f', 'webm',
                    '-y',
                    temp_path.resolve()]
    
    try:
        subprocess.run(full_command, check=True, stdin=media.open())
    except BaseException as e:
        temp_path.unlink(missing_ok=True)
        raise e
    
    if temp_path.stat().st_size > media.size():
        temp_path.unlink()
        logger.warning(f"Transcoding did not reduce file size for {file_path}, keeping original")
        return
    
    paths_by_inode: dict[int, List[Path]] = {}
    for file_path in media.paths:
        media_directory = file_path.parent
        bin(file_path, recycle_bin)
        new_media = shutil.move(temp_path, media_directory.joinpath(os.path.splitext(file_path.name)[0] + ".webm"))
        inode = new_media.stat().st_ino
        if inode in paths_by_inode:
            paths_by_inode[inode].append(new_media)
        else:
            paths_by_inode[inode] = [new_media]
    logger.info(f"Transcoded {media.paths} to {paths_by_inode.values()}")
    return [MediaDTO(inode, paths, True) for inode, paths in paths_by_inode.items()]

def bin(file: Path, recycle_bin: Path | None):
    if recycle_bin:
        recycle_bin.mkdir(parents=True, exist_ok=True)
        shutil.move(file, recycle_bin.joinpath(file.name))
    else:
        file.unlink()
